import os
import numpy as np
import torch
from timer import Timer
from ImageDataLoader import ImageDataLoader
from network import Network
from evaluate_model import evaluate_model
import cv2
import math

try:
    from termcolor import cprint
except ImportError:
    cprint = None

def log_print(text, color=None, on_color=None, attrs=None):
    if cprint is not None:
        cprint(text, color=color, on_color=on_color, attrs=attrs)
    else:
        print(text)
def save_net(fname, net):
    import h5py
    h5f = h5py.File(fname, mode='w')
    for k, v in net.state_dict().items():
        h5f.create_dataset(k, data=v.cpu().numpy())

def train( nIndex, traindata_name, validdata_name ):

    index        = nIndex
    method       = 'Alexnet'
    dataset_name = 'Geomorphology'
    name         = validdata_name.replace( '.txt','' )
    output_dir   = '/home/leequer/like/model/'+ name  + '/model' + str(nIndex) + '/'    

    print( 'output_dir',output_dir)

    train_path       = '/home/leequer/pytorch/new_data/'
    traindata_loader = ImageDataLoader(train_path, traindata_name, shuffle=True, imresize=True, pre_load=True)
    valid_path       = train_path  
    valdata_loader   = ImageDataLoader(valid_path, validdata_name, shuffle=True, imresize=True, pre_load=True)
     

    net = Network(index)
    net.cuda()
    net.train()

    #training configuration
    start_step = 0
    end_step   = 5000
    lr         = 0.00001
    momentum   = 0.9


    # ------------
    rand_seed = 64678  
    if rand_seed is not None:
        np.random.seed(rand_seed)
        torch.manual_seed(rand_seed)
        torch.cuda.manual_seed(rand_seed)

    
    params = list(net.parameters())
    optimizer = torch.optim.Adam(filter(lambda p: p.requires_grad, net.parameters()), lr=lr)


    # training
    train_loss = 0
    step_cnt = 0
    re_cnt = False
    t = Timer()
    t.tic()


    batchSize = 50
    dataCount = len(traindata_loader) 

    best = 0

    best_model = ''

    for epoch in range(start_step, end_step+1):    
        step = -1
        train_loss = 0
        idx = 0 

        im_data         = [] 
        gt_data         = []
        slope_data      = []
        curvation_data  = [] 
        img_data        = []
        wetidx_data     = [] 
        flowacc_data    = [] 


        for blob in traindata_loader:                
            step      = step + 1        
            data      = blob['data']
            dem       = blob['dem']
            slope     = blob['slope']
            img       = blob['imgg']            
            label     = blob['label']

            if idx==0:
               im_data        = data 
               dem_data       = dem
               slope_data     = slope
               img_data       = img
               #curvation_data = curvation
               #wetidx_data    = wetidx
               #flowacc_data   = flowacc

               gt_data = np.array( label) 
               gt_data = gt_data.reshape( 1, 1 )
            else:
               im_data        = np.concatenate([im_data,data],axis=0)
               dem_data       = np.concatenate([dem_data,dem],axis=0)
               slope_data     = np.concatenate([slope_data,slope],axis=0)
               #curvation_data = np.concatenate([curvation_data,curvation],axis=0)
               #flowacc_data   = np.concatenate([flowacc_data,flowacc],axis=0)
               #wetidx_data    = np.concatenate([wetidx_data,wetidx],axis=0)
               img_data       = np.concatenate([img_data,img],axis=0)
               

               label = np.array( label) 
               label = label.reshape( 1, 1 )
               gt_data =np.concatenate([gt_data,label],axis=0)

            idx = idx+1
            if idx<batchSize:
                if step<dataCount-1:
                   continue

            pre_label = net(im_data,dem_data, slope_data, curvation_data,img_data,wetidx_data,flowacc_data,index ,gt_data)
            loss = net.loss

            train_loss += loss.item()  
            step_cnt += 1
            optimizer.zero_grad()
            loss.backward()
            optimizer.step()
            idx = 0

        if epoch % 10 == 0:            
            duration = t.toc(average=False)
            fps = step_cnt / duration
            log_text = 'index: %4d, epoch: %4d, step %4d, Time: %.4fs, Duration: %.4fs, loss: %4.4f' % (index, epoch, step, 1./fps,duration, train_loss)

            log_print(log_text, color='green', attrs=['bold'])
            re_cnt = True    

           
        if re_cnt:                                
            t.tic()
            re_cnt = False
                 
        if (epoch % 200 == 0):
            save_name = os.path.join(output_dir, '{}_{}_{}.h5'.format(method,dataset_name,epoch))
            save_net(save_name, net)     
            #calculate error on the validation dataset 
            precision = evaluate_model(save_name, valdata_loader,20,index)
            if precision > best:
                best = precision
                best_model = '{}_{}_{}.h5'.format(method,dataset_name,epoch)
            log_text = 'EPOCH: %d, precision: %.4f' % (epoch,precision)
            log_print(log_text, color='green', attrs=['bold'])
            log_text = 'BEST Precision: %0.4f, BEST MODEL: %s' % (best, best_model)
            log_print(log_text, color='green', attrs=['bold'])    

    return best_model    



